@@ -9,8 +9,8 @@ from onnx.reference.ops._op import OpRunUnaryNum
 
 
 class HardSigmoid(OpRunUnaryNum):
-    def _run(self, x, alpha=None, beta=None):  # type: ignore
-        alpha = alpha or self.alpha  # type: ignore
-        beta = beta or self.beta  # type: ignore
-        y = np.maximum(0, np.minimum(1, x * alpha + beta))
+    def _run(self, x, alpha=None, beta=None):
+        alpha = alpha or self.alpha
+        beta = beta or self.beta
+        y = np.maximum(0, np.minimum(1, x * alpha + beta)).astype(x.dtype)
         return (y,)
