@@ -11,7 +11,7 @@ from onnx.reference.op_run import OpRun
 
 
 class Pow(OpRun):
-    def _run(self, a, b):  # type: ignore
+    def _run(self, a, b):
         with catch_warnings():
             simplefilter("ignore")
             return (np.power(a, b).astype(a.dtype),)
